//
//  XMPPPresence.m
//  Jabber
//
//  Created by David Chisnall on Sun Apr 25 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import <EtoileXML/ETXMLNode.h>
#import <EtoileXML/ETXMLParser.h>
#import <EtoileFoundation/EtoileCompatibility.h>
#import "XMPPPresence.h"
#import "XMPPPresenceStanzaFactory.h"

int PRESENCE_ICONS[] = {
	0x25C9,
	0x25C9,
	0x25C8,
	0x25C8,
	0x25A3,
	0x25CE,
	0x25CE
};


@implementation XMPPPresence
+ (NSString*) displayStringForPresence:(unsigned char)_presence
{
	switch(_presence)
	{
		case PRESENCE_CHAT:
			return @"Free For Chat";
		case PRESENCE_ONLINE:
			return @"Online";
		case PRESENCE_AWAY:
			return @"Away";
		case PRESENCE_XA:
			return @"Extended Away";
		case PRESENCE_DND:
			return @"Do Not Disturb";
		case PRESENCE_OFFLINE:
			return @"Offline";
	}	
	return @"Unknown";
}

+ (NSString*) xmppStringForPresence:(unsigned char)_presence
{
	switch(_presence)
	{
		case PRESENCE_CHAT:
			return @"chat";
		case PRESENCE_ONLINE:
			return @"online";
		case PRESENCE_AWAY:
			return @"away";
		case PRESENCE_XA:
			return @"xa";
		case PRESENCE_DND:
			return @"dnd";
	}	
	return @"unknown";
}

+ (unsigned char) presenceForXMPPString:(NSString*)_presence
{
	if([_presence rangeOfString:@"online"].location != NSNotFound)
	{
		return PRESENCE_ONLINE;
	}
	if([_presence rangeOfString:@"away"].location != NSNotFound)
	{
		return PRESENCE_AWAY;
	}
	if([_presence rangeOfString:@"xa"].location != NSNotFound)
	{
		return PRESENCE_XA;
	}
	if([_presence rangeOfString:@"dnd"].location != NSNotFound)
	{
		return PRESENCE_DND;
	}
	if([_presence rangeOfString:@"offline"].location != NSNotFound)
	{
		return PRESENCE_OFFLINE;
	}
	if([_presence rangeOfString:@"chat"].location != NSNotFound)
	{
		return PRESENCE_CHAT;
	}
	return PRESENCE_UNKNOWN;
}

- (void)startElement:(NSString *)aName
		  attributes:(NSDictionary *)attributes
{
	if ([aName isEqualToString:@"presence"])
	{
		depth++;
		from = [JID jidWithString:[attributes objectForKey:@"from"]];
		NSString * presenceType = [attributes objectForKey:@"type"];
		priority = 0;
		onlineStatus = PRESENCE_UNKNOWN;
#ifndef DNDEBUG
		ETLog(@"Presence from %@: %@", [attributes objectForKey:@"from"], presenceType);
#endif
		if(presenceType == nil || [presenceType isEqualToString:@"online"])
		{
#ifndef DNDEBUG
			ETLog(@"Online");
#endif
			onlineStatus = PRESENCE_ONLINE;
			type = online;
		}
		else if([presenceType isEqualToString:@"unavailable"])
		{
			type = unavailable;
			onlineStatus = PRESENCE_OFFLINE;
		}
		else if([presenceType isEqualToString:@"subscribe"])
		{
			type = subscribe;
		}
		else if([presenceType isEqualToString:@"unsubscribe"])
		{
			type = unsubscribe;
		}
		else if([presenceType isEqualToString:@"subscribed"])
		{
			type = subscribed;
		}
		else if([presenceType isEqualToString:@"unsubscribed"])
		{
			type = unsubscribed;
		}
		else
		{
			//ETLog(@"Presence stanza with unknown type received.");
		}
	}
	else if([aName isEqualToString:@"c"])
	{
		caps = [attributes objectForKey:@"ver"];
		depth++;
	}
	else
	{
		XMPPPresenceStanzaFactory * factory = [XMPPPresenceStanzaFactory sharedStazaFactory];
		NSString * xmlns = [attributes objectForKey:@"xmlns"];
		Class handler = [factory handlerForTag:aName inNamespace:xmlns];
		NSString * elementKey = [factory valueForTag:aName inNamespace:xmlns];
		[[[handler alloc] initWithXMLParser:parser
						key:elementKey] startElement:aName														         attributes:attributes];
		
	}
}

// Sub-stanza KVC handlers
- (void) addshow:(NSString*)show
{
	onlineStatus = [XMPPPresence presenceForXMPPString:show];			
}
- (void) addstatus:(NSString*)status
{
	message = status;
}
- (void) setpriority:(NSString*)aPriority
{
	priority = [aPriority intValue];
}
- (void) addnickname:(NSString*)aNickname
{
	nickname = aNickname;
}

- (id) initWithJID:(JID*)_jid
{
	from = _jid;
	onlineStatus = PRESENCE_UNKNOWN;
	message = @"Status Unknown";
	return [super init];
}


- (id) initWithXMLParser: (ETXMLParser*)aParser
                     key: (id) aKey
{
	self = [super initWithXMLParser: aParser
	                            key: aKey];
	if (nil == self)
	{
		return nil;
	}
	from = [[JID alloc] init];
	onlineStatus = PRESENCE_UNKNOWN;
	message = nil;
	return self;
}

- (unsigned char) show
{
	return onlineStatus;
}

- (NSString*) status
{
	return message;
}

- (NSString*) nickname
{
	return nickname;
}

- (JID*) jid
{
	return from;
}

- (int) priority
{
	return priority;
}

- (PresenceType) type
{
	return type;
}

- (NSString*) caps
{
	return caps;
}

- (NSComparisonResult) compare:(XMPPPresence*)_otherPresence
{
	if(onlineStatus < [_otherPresence show])
	{
		return NSOrderedAscending;
	}
	if(onlineStatus > [_otherPresence show])
	{
		return NSOrderedDescending;
	}
	return NSOrderedSame;
}

@end
